#include <stdio.h>			/* Standard I/I */
#include "../simlib.h"
#define MPE_GRAPHICS
#include "mpe.h"

#define WINDOW_SIZE 300

void UserData ANSI_ARGS((SIM_Ctx *)); /* Callback prototype */
void simple_draw();
void init_shared_data();

#define BUFF_SIZE 500000
typedef struct { float x, y, z ; } Vertex;
struct global_mem {
    int ix;
    Vertex v[BUFF_SIZE];
} *gm;

main( argc, argv )
int  argc;
char **argv;
{

    int rc;
    MPE_XGraph Graph;
    int x, y, button, pressed;
    SIM_Ctx *simctx;

    init_shared_data();

    while (1) {
	/* Register and wait for connections */
	simctx = SIM_Contact( "xfip", "monte" );
	if (!simctx) {
	    printf( "Could not contact monte\n" );
	    exit( 1 );
	}
	SIM_ReadCallback( simctx, UserData );

	/* Link with MPI-less MPE version */
	MPE_Open_graphics( &Graph, 0, (char *)0, -1, -1, 
			   WINDOW_SIZE, WINDOW_SIZE, 0 );
	gm->ix = 0;
    
	/* Read data as it becomes available and draw it.  Send data back
	   to the partner if available */
	/* Note that the RIGHT way to do this is to add the data FD 
	   to the X loop or alternately add the X fd to OUR handling */
	printf( "Starting event loop\n" );
	while (1) {
	    /* Needs nonblocking check for escape key in window */

	    MPE_Iget_mouse_press( Graph, &x, &y, &button, &pressed );
	    if (pressed) {
		/* Send button to partner */
		SIM_SendInts( simctx, &button, 1 );
		if (button == MPE_BUTTON2) break;
	    }
	    /* Check for data and redraw */
	    SIM_Update( simctx );
	    simple_draw( Graph );
	}

	MPE_Close_graphics( &Graph );
	SIM_Close( simctx );
    }
    
}

void simple_draw( Graph )
MPE_XGraph Graph;
{
    int i;
    int x, y, w=10, h=10;

    /* Clear background first */
    MPE_Fill_rectangle( Graph, 0, 0, WINDOW_SIZE, WINDOW_SIZE, MPE_WHITE );
    /* printf( "Drawing %d rectangles\n", gm->ix ); */
    for (i = 0; i< gm->ix; i++) {
	x = (gm->v[i].x + 1) * WINDOW_SIZE/2;
	y = (gm->v[i].y + 1) * WINDOW_SIZE/2;
	/* printf( "rectangle centered at (%d,%d)\n", x,y ); */
	MPE_Fill_rectangle( Graph, x - w/2, y - h/2, w, h, MPE_RED );
    }
    MPE_Update( Graph );
}

/*
  This is the callback called when the stream data is received
*/
void UserData( simctx )
SIM_Ctx *simctx;
{
    /* printf( "Reading data\n" ); */
    SIM_ReadFloats( simctx, &gm->v[gm->ix], 3 );
    if (++gm->ix == BUFF_SIZE) {
	printf("Exceeded buff size \n");
	exit();
    }

    /*
      printf("                 double %f, %f, %f\n",xyz[0], xyz[1], xyz[2]);
      printf("from source %d\n",id);
      */
} 

void init_shared_data()
{
    gm = (struct global_mem *) malloc(sizeof(struct global_mem));
    if (gm == NULL) {
	printf("Error in gm amalloc\n");
	exit(0);
    }

    gm->ix = 0;
    printf("After data init\n");
}


