#include "srlist.h"
#include <string.h>

/* 
 * This is a VERY simple search system, based on hashed lists.
 * Experts or people with time on their hands can write something better.
 * This is good enough, however, for most of what we need here.
 */

/* Private functions */
static int HashFn( SrList *self, char *name )
{
    int idx = 0;
    while (*name) idx += *name++;
    idx = idx % self->hash_size;
    return idx;
}

/* Public functions */
static void Destroy( SrList *self )
{
    int i;
    SrEntry *p, *n;

    for (i=0; i<self->hash_size; i++) {
	if ((p = self->table[i])) {
	    while (p) {
		n = p->next;
		free( p );
		p = n;
		}
	    }
	}
    free( self->table );
}

static Lookup( SrList *self, char *name, SrEntry **val )
{
    int     idx;
    SrEntry *p;

    idx = (self->HashFn)( self, name );
    p   = self->table[idx];
    while (p) {
	if (strcmp( p->name, name ) == 0) {
	    *val = p;
	    return 0;
	    }
	p = p->next;
	}
    return 1;
}

static int Insert( SrList *self, char *name, SrEntry **val )
{
    int     idx;
    SrEntry *p;

    idx = (self->HashFn)( self, name );
    p   = self->table[idx];
    while (p) {
	if (strcmp( p->name, name ) == 0) {
	    *val = p;
	    return 0;
	    }
	p = p->next;
	}
    /* Did not find an entry.  Create and return */
    p		  = (SrEntry *)malloc( sizeof(SrEntry) );
    p->next	  = self->table[idx];
    p->name	  = (char *)malloc( strlen(name)+1 );
    p->extra_data = 0;
    strcpy( p->name, name );

    self->table[idx] = p;
    *val = p;
    return 0;

}

static int Delete( SrList *self, char *name )
{
    int i;
    SrEntry *p, *n;
    
    i = (self->HashFn)( self, name );
    p = self->table[i];
    n = 0;
    while (p) {
	if (strcmp( p->name, name ) == 0) {
	    if (n)
		n->next = p->next;
	    else
		self->table[i] = p->next;
	    free( p );
	    }
	n = p;
	p = p->next;
	}
    return 0;
}

static int Walk( SrList *self, int (*fcn)( char *, SrEntry *, void * ),
		 void *fcn_data )
{
    int i;
    SrEntry *p;
    
    for (i=0; i<self->hash_size; i++) {
	p = self->table[i];
	while (p) {
	    fcn( p->name, p, fcn_data );
	    p = p->next;
	    }
	}
    return 0;
}

SrList *NewSrList( int h_size )
{
    int i;
    SrList    *new;

    new = (SrList *) malloc( sizeof( SrList ) );
    new->hash_size = h_size;
    new->table     = (SrEntry **) malloc( sizeof(SrEntry *)*h_size );
    for (i=0; i<h_size; i++) 
	new->table[i] = 0;
    new->HashFn	 = HashFn;
    new->destroy = Destroy;
    new->Lookup	 = Lookup;
    new->Insert	 = Insert;
    new->Delete	 = Delete;
    new->Walk	 = Walk;
    
    return new;
}

