/*
 * This file contains a simple set of monitor functions that can be 
 * installed on fd's
 */

#include "pexec.h"
#include <stdio.h>

int PX_Hfcn_Echo_action( ctx, conn, handler, in_mask )
PX_Ctx     *ctx;
PX_Conn    *conn;
PX_Handler handler;
int        in_mask;
{
char *p = 0;
char *which = (char *)handler->extra_data;
int  err;

if (handler->next && handler->next->info) {
    (*handler->next->info)( handler->next, &p );
    }
if (p) {
    printf( "Calling %s (%s) on fd %d, mask %x\n", 
	    which, p, conn->fd, in_mask );
    }
else {
    printf( "Calling %s on fd %d, mask %x\n", which, conn->fd, in_mask );
    }
fflush( stdout );
if (handler->next == handler) {
    printf( "Error - circular handlers!\n" );
    return 0;
    }
if (handler->next) {
    err = (*handler->next->fcn)( ctx, conn, handler->next, in_mask );
    }
printf( "Finished handler on fd %d\n", conn->fd );
fflush( stdout );
return err;
}

/*@
   PX_Monitor_setup - Installs a simple tracing handler to all fds

   Input Parameter:
.  ctx - Pexec context

   Notes:
   The handlers installed by this call are placed at the head of each
   possible fd, and are persistent (they are not deleted when the fd is
   closed).  The handlers write some information about the handlers that
   are actually being used to handle the data to standard output.

   This is an example of composing fd handlers to provide extra debugging
   and monitoring information.
@*/
int PX_Monitor_setup( ctx )
PX_Ctx *ctx;
{
int        i;
PX_Conn    *conn;
PX_Handler do_echo;
static char s1[10] = "handler",
            s2[10] = "on_err",
            s3[10] = "on_eof";

for (i=0; i<FD_SETSIZE; i++) {
    conn		= ctx->conn + i;
    do_echo = PX_Handler_simple_create( PX_Hfcn_Echo_action, (void *)s1 );
    do_echo->disposition = PX_KEEP_ON_CLOSE;
    conn->handler	= do_echo;
    do_echo->ref_count ++;
    PX_Handler_free( do_echo );
    do_echo = PX_Handler_simple_create( PX_Hfcn_Echo_action, (void *)s2 );
    do_echo->disposition = PX_KEEP_ON_CLOSE;
    conn->on_err	= do_echo;
    do_echo->ref_count ++;
    PX_Handler_free( do_echo );
    do_echo = PX_Handler_simple_create( PX_Hfcn_Echo_action, (void *)s3 );
    do_echo->disposition = PX_KEEP_ON_CLOSE;
    conn->on_eof	= do_echo;
    do_echo->ref_count ++;
    PX_Handler_free( do_echo );
    }

return 0;
}
