#include "pexec.h"

/*@
  PX_Util_Deltatime - Returns a time in the future 

  Input Parameters:
. seconds - Number of seconds in the future
. usec    - Number of microseconds in the future
. tval    - Pointer to structure to use for timeval.

  Returns:
  Pointer to timeval structure for the specified time in the future.
  This in in absolute time format, as required by PEXEC routines.
  The pointer is 'tval' if 'tval' is non-null; otherwise it is a `local static`
  variable and should not be used if there is any chance that this 
  data will be overwritten.

  Note that when used with 'PX_FD_Add', it is ok to use a null 'tval' 
  because 'PX_FD_Add' makes its own copy of the timevalue.
@*/
struct timeval *PX_Util_Deltatime( seconds, usec, tval )
int            seconds, usec;
struct timeval *tval;
{
static struct timeval mtval;

if (!tval) tval = &mtval;
PX_Gettimeofday(tval,(void *)0);
tval->tv_sec  += seconds;
tval->tv_usec += usec;
if (tval->tv_usec > 1000000) {
    tval->tv_sec++;
    tval->tv_usec -= 1000000;
    }
return tval;
}
